@@ -107,6 +107,23 @@ static void testmode(int mode)
 	    vga_setrgbcolor(0, 0, c);
 	    vga_drawline(x, y, x, y + yw - 1);
 	}
+	for (x = 0; x < 64; x++) {
+	    for (y = 0; y < 64; y++) {
+		vga_setrgbcolor(x * 4 + 3, y * 4 + 3, 0);
+		vga_drawpixel(xmax / 2 - 160 + x, y + ymax / 2 - 80);
+		vga_setrgbcolor(x * 4 + 3, 0, y * 4 + 3);
+		vga_drawpixel(xmax / 2 - 32 + x, y + ymax / 2 - 80);
+		vga_setrgbcolor(0, x * 4 + 3, y * 4 + 3);
+		vga_drawpixel(xmax / 2 + 160 - 64 + x, y + ymax / 2 - 80);
+
+		vga_setrgbcolor(x * 4 + 3, y * 4 + 3, 255);
+		vga_drawpixel(xmax / 2 - 160 + x, y + ymax / 2 + 16);
+		vga_setrgbcolor(x * 4 + 3, 255, y * 4 + 3);
+		vga_drawpixel(xmax / 2 - 32 + x, y + ymax / 2 + 16);
+		vga_setrgbcolor(255, x * 4 + 3, y * 4 + 3);
+		vga_drawpixel(xmax / 2 + 160 - 64 + x, y + ymax / 2 + 16);
+	    }
+	}
 	break;
     default:
 	if (vga_getcolors() == 16) {
