@@ -86,7 +86,7 @@
 #define ARK1000PV	0
 #define ARK2000PV	1
 
-#define VESA	0
+#define VL	0
 #define PCI	1
 
 static int ark_chip;
@@ -555,7 +555,7 @@ static int ark_linear(int op, int param)
 	if (ark_bus == PCI && param == 0)
 	    /* Return PCI base address. */
 	    return ark_baseaddress;
-	if (ark_bus == VESA) {
+	if (ark_bus == VL) {
 	    if (param == 0)
 		return 0x08000000;	/* 128MB */
 	    if (param == 1)
@@ -600,7 +600,8 @@ DriverSpecs __svgalib_ark_driverspecs =
     0,				/* extset */
     0,
     ark_linear,
-    NULL			/* accelspecs */
+    NULL,			/* accelspecs */
+    NULL,                       /* Emulation */
 };
 
 /* ARK-specific config file options. */
@@ -795,7 +796,7 @@ static int ark_init(int force, int par1, int par2)
     if (__svgalib_driver_report) {
 	char *bustype;
 	if (__svgalib_inSR(0x19) & 0x80) {
-	    ark_bus = VESA;
+	    ark_bus = VL;
 	    bustype = "VL bus";
 	} else {
 	    ark_bus = PCI;
