@@ -2069,7 +2069,8 @@ DriverSpecs __svgalib_chips_driverspecs =
 	0,				/* extset */
 	0,
 	CHIPS_linear,			/* linear */
-	NULL				/* accelspecs */
+	NULL,				/* Accelspecs */
+	NULL,				/* Emulation */
 };
 
 /* Chips and Technologies specific config file options.
