@@ -886,7 +886,8 @@ DriverSpecs __svgalib_cirrus_driverspecs =
     0,				/* ext_set */
     0,				/* accel */
     cirrus_linear,
-    0				/* accelspecs, filled in during init. */
+    NULL,                       /* Accelspecs */
+    NULL,                       /* Emulation */
 };
 
 
