@@ -635,7 +635,7 @@ static int et4000_setmode(int mode, int prv_mode)
 	outb(__svgalib_CRT_I, 0x34);
 	i = inb(__svgalib_CRT_D);
 	if ((i & 0x02) == 0x02)
-	    outb(__svgalib_CRT_D, (i & 0xfe));
+	    outb(__svgalib_CRT_D, (i & 0xfd)); /* DS */
 	return __svgalib_vga_driverspecs.setmode(mode, prv_mode);
     case SVGADRV:
 	regs = LOOKUPMODE(et4000_modes, mode);
@@ -905,7 +905,8 @@ DriverSpecs __svgalib_et4000_driverspecs =
     et4000_ext_set,
     0,
     et4000_linear,
-    NULL			/* accelspecs */
+    NULL,                       /* Accelspecs */
+    NULL,                       /* Emulation */
 };
 
 
