@@ -274,7 +274,8 @@ DriverSpecs __svgalib_gvga6400_driverspecs =
     0,				/* extset */
     0,
     0,				/* linear */
-    NULL			/* accelspecs */
+    NULL,                       /* Accelspecs */
+    NULL,                       /* Emulation */
 };
 
 /* Initialize chipset (called after detection) */
