@@ -146,6 +146,7 @@ extern int COL;			/* current color            */
 extern int CM;			/* current video mode       */
 extern struct info infotable[];
 extern int SCREENON;		/* screen visible if != 0 */
+extern unsigned long __svgalib_graph_base;
 extern unsigned char *GM;	/* graphics memory frame */
 #ifdef __alpha__
 extern unsigned char *SM;	/* sparse graphics memory frame */
@@ -155,10 +156,19 @@ extern int MODEFLAGS;		/* copy of flags of current modeinfo->flags */
 
 extern int __svgalib_mem_fd;
 extern int __svgalib_tty_fd;
+extern int __svgalib_nosigint;
 extern int __svgalib_mouse_fd;
 extern int __svgalib_kbd_fd;
 extern int __svgalib_runinbackground;
 
+extern unsigned char __svgalib_novga;
+extern unsigned char __svgalib_secondary;
+extern unsigned char __svgalib_m_ignore_dx;
+extern unsigned char __svgalib_m_ignore_dy;
+extern unsigned char __svgalib_m_ignore_dz;
+
+extern char *__joystick_devicenames[4];
+
 /* --------------------- Function definitions shared by library modules */
 
 extern int __svgalib_setregs(const unsigned char *regs);
@@ -173,6 +183,10 @@ extern int __svgalib_physmem(void);
 extern void __svgalib_waitvtactive(void);
 extern void __svgalib_open_devconsole(void);
 extern void __svgalib_flipaway(void);
+extern void (*__svgalib_mouse_eventhandler) (int, int, int, int, int, int, int);
+extern void (*__svgalib_keyboard_eventhandler) (int, int);
+extern void __joystick_flip_vc(int acquire);
+
 
 #if 0 
 /* remove this part ? */
