@@ -7,7 +7,8 @@
 /* Multi-chipset support Copyright (C) 1993 Harm Hanemaayer */
 /* partially copyrighted (C) 1993 by Hartmut Schirmer */
 /* Changes by Michael Weller. */
-
+/* Modified by Don Secrest to include Tseng ET6000 handling */
+/* Changes around the config things by 101 (Attila Lendvai) */
 
 /* The code is a bit of a mess; also note that the drawing functions */
 /* are not speed optimized (the gl functions are much faster). */
@@ -70,6 +71,8 @@ int __svgalib_CRT_D;			/* current CRT data register address */
 int __svgalib_IS1_R;			/* current input status register address */
 static int color_text;		/* true if color text emulation */
 
+/* If == 0 then nothing is defined by the user... */
+int __svgalib_default_mode = 0;
 
 struct info infotable[] =
 {
@@ -151,6 +154,10 @@ struct info infotable[] =
 
 #define MAX_MODES (sizeof(infotable) / sizeof(struct info))
 
+unsigned long __svgalib_graph_base = GRAPH_BASE;
+
+unsigned char __svgalib_novga = 0;     /* Does not have VGA circuitry on board */
+unsigned char __svgalib_secondary = 0; /* this is not the main card with VC'S (not yet supported) */
 
 /* default palette values */
 static const unsigned char default_red[256]
@@ -276,6 +283,7 @@ static int modeinfo_mask = ~0;
 
 int __svgalib_mem_fd = -1;	/* /dev/mem file descriptor  */
 int __svgalib_tty_fd = -1;	/* /dev/tty file descriptor */
+int __svgalib_nosigint = 0;	/* Don't generate SIGINT in graphics mode */
 int __svgalib_runinbackground = 0;
 static int svgalib_vc = -1, startup_vc = -1;
 static int __svgalib_security_revokeallprivs = 1;
@@ -416,7 +424,27 @@ DriverSpecs *__svgalib_driverspecslist[] =
     NULL,
 #endif
 #ifdef INCLUDE_CHIPS_DRIVER
-    &__svgalib_chips_driverspecs
+    &__svgalib_chips_driverspecs,
+#else
+    NULL,
+#endif
+#ifdef INCLUDE_APM_DRIVER
+    &__svgalib_apm_driverspecs,
+#else
+    NULL,
+#endif
+#ifdef INCLUDE_NV3_DRIVER
+    &__svgalib_nv3_driverspecs,
+#else
+    NULL,
+#endif
+#ifdef INCLUDE_ET6000_DRIVER
+    &__svgalib_et6000_driverspecs,
+#else
+    NULL,
+#endif
+#ifdef INCLUDE_VESA_DRIVER
+    &__svgalib_vesa_driverspecs
 #else
     NULL
 #endif
@@ -424,7 +452,8 @@ DriverSpecs *__svgalib_driverspecslist[] =
 
 static char *driver_names[] =
 {"", "VGA", "ET4000", "Cirrus", "TVGA", "Oak", "EGA", "S3",
- "ET3000", "Mach32", "GVGA6400", "ARK", "ATI", "ALI", "Mach64", "C&T", NULL};
+ "ET3000", "Mach32", "GVGA6400", "ARK", "ATI", "ALI", "Mach64", 
+ "C&T", "APM", "NV3", "ET6000", "VESA", NULL};
 
 /* Chipset drivers */
 
@@ -442,6 +471,7 @@ static char *driver_names[] =
 /* ali          ALI2301 */
 /* mach64	ATI MACH64 */
 /* chips	chips & technologies*/
+/* et6000       Tseng ET6000 */         /* DS */
 
 /*#define DEBUG */
 
@@ -490,6 +520,8 @@ static void disable_interrupt(void)
 {
     struct termios cur_termio;
 
+    /* Well, one could argue that sigint is not enabled at all when in __svgalib_nosigint
+       but sometimes they *still* are enabled b4 graph_termio is set.. */
     ioctl(0, TCGETS, &cur_termio);
     cur_termio.c_lflag &= ~ISIG;
     ioctl(0, TCSETSW, &cur_termio);
@@ -500,6 +532,8 @@ static void enable_interrupt(void)
 {
     struct termios cur_termio;
 
+    if (__svgalib_nosigint) /* do not reenable, they are often reenabled by text_termio */
+	return; 
     ioctl(0, TCGETS, &cur_termio);
     cur_termio.c_lflag |= ISIG;
     ioctl(0, TCSETSW, &cur_termio);
@@ -683,7 +717,7 @@ void __svgalib_get_perm(void)
     __svgalib_open_devconsole();
 
     /* color or monochrome text emulation? */
-    if (CHIPSET != EGA)
+    if (CHIPSET != EGA && !__svgalib_novga)
 	color_text = port_in(MIS_R) & 0x01;
     else
 	color_text = 1;		/* EGA is assumed color */
@@ -761,7 +795,7 @@ int __svgalib_saveregs(unsigned char *regs)
 {
     int i;
 
-    if (__svgalib_chipset == EGA) {
+    if (__svgalib_chipset == EGA || __svgalib_novga) {
 	/* Special case: Don't save standard VGA registers. */
 	return chipset_saveregs(regs);
     }
@@ -803,6 +837,8 @@ int __svgalib_setregs(const unsigned char *regs)
 {
     int i;
 
+    if(__svgalib_novga) return 1;
+
     if (__svgalib_chipset == EGA) {
 	/* Enable graphics register modification */
 	port_out(0x00, GRA_E0);
@@ -880,8 +916,10 @@ static void restoretextmode(void)
     if (!__svgalib_screenon)
 	vga_screenon();
     if (__svgalib_tty_fd >= 0) {
-	ioctl(__svgalib_tty_fd, KDSETMODE, KD_TEXT);
-        ioctl(__svgalib_tty_fd, VT_SETMODE, &__svgalib_oldvtmode);
+        if (!__svgalib_secondary) {
+	    ioctl(__svgalib_tty_fd, KDSETMODE, KD_TEXT);
+            ioctl(__svgalib_tty_fd, VT_SETMODE, &__svgalib_oldvtmode);
+	}
     }
 }
 
@@ -947,6 +985,11 @@ int __svgalib_getchipset(void)
 	    CHIPSET = EGA;
 	else
 #endif
+#ifdef INCLUDE_ET6000_DRIVER_TEST                   /* DS */
+	if (__svgalib_et6000_driverspecs.test())    /* This must be before */
+	    CHIPSET = ET6000;                       /* ET4000 or the card  */
+	else                                        /* will be called et4k */
+#endif
 #ifdef INCLUDE_ET4000_DRIVER_TEST
 	if (__svgalib_et4000_driverspecs.test())
 	    CHIPSET = ET4000;
@@ -998,6 +1041,23 @@ int __svgalib_getchipset(void)
 	    CHIPSET = ALI;
 	else
 #endif
+#ifdef INCLUDE_NV3_DRIVER_TEST
+	if (__svgalib_nv3_driverspecs.test())
+	    CHIPSET = NV3;
+	else
+#endif
+#ifdef INCLUDE_APM_DRIVER_TEST
+/* Note: On certain cards this may toggle the video signal on/off which
+   is ugly. Hence we test this last. */
+	if (__svgalib_apm_driverspecs.test())
+	    CHIPSET = APM;
+	else
+#endif
+#ifdef INCLUDE_VESA_DRIVER_TEST
+	if (__svgalib_vesa_driverspecs.test())
+	    CHIPSET = VESA;
+	else
+#endif
 
 	if (__svgalib_vga_driverspecs.test())
 	    CHIPSET = VGA;
@@ -1056,24 +1116,12 @@ static void savepalette(unsigned char *red, unsigned char *green,
 {
     int i;
 
-    if (CHIPSET == EGA)
+    if (__svgalib_driverspecs->emul && __svgalib_driverspecs->emul->savepalette) 
+        return (__svgalib_driverspecs->emul->savepalette(red, green, blue));
+
+    if (CHIPSET == EGA || __svgalib_novga) 
 	return;
 
-    if ((__svgalib_chipset == MACH32) && SVGAMODE(CM)) {
-	/* Actually the same but we are in 8514 mode and the dac
-	   does not respond to the VGA circuitry anymore... */
-
-	port_out(0, PEL8514_IR);
-	for (i = 0; i < 256; i++) {
-	    __svgalib_delay();
-	    *(red++) = port_in(PEL8514_D);
-	    __svgalib_delay();
-	    *(green++) = port_in(PEL8514_D);
-	    __svgalib_delay();
-	    *(blue++) = port_in(PEL8514_D);
-	}
-	return;
-    }
     /* save graphics mode palette - first select palette index 0 */
     port_out(0, PEL_IR);
 
@@ -1093,24 +1141,12 @@ static void restorepalette(const unsigned char *red,
 {
     int i;
 
-    if (CHIPSET == EGA)
+    if (__svgalib_driverspecs->emul && __svgalib_driverspecs->emul->restorepalette) 
+        return (__svgalib_driverspecs->emul->restorepalette(red, green, blue));
+
+    if (CHIPSET == EGA || __svgalib_novga)
 	return;
 
-    if ((__svgalib_chipset == MACH32) && SVGAMODE(CM)) {
-	/* Actually the same but we are in 8514 mode and the dac
-	   does not respond to the VGA circuitry anymore... */
-
-	port_out(0, PEL8514_IW);
-	for (i = 0; i < 256; i++) {
-	    __svgalib_delay();
-	    port_out(*(red++), PEL8514_D);
-	    __svgalib_delay();
-	    port_out(*(green++), PEL8514_D);
-	    __svgalib_delay();
-	    port_out(*(blue++), PEL8514_D);
-	}
-	return;
-    }
     /* restore saved palette */
     port_out(0, PEL_IW);
 
@@ -1306,7 +1342,7 @@ static void __vga_mmap(void)
 						  PROT_READ | PROT_WRITE,
 						  MAP_SHARED | MAP_FIXED,
 						    __svgalib_mem_fd,
-						    GRAPH_BASE
+						    __svgalib_graph_base
 	);
 #endif
 #if 1
@@ -1319,7 +1355,7 @@ static void __vga_mmap(void)
 				   PROT_READ | PROT_WRITE,
 				   MAP_SHARED,
 				   __svgalib_mem_fd,
-				   GRAPH_BASE
+				   __svgalib_graph_base
 	);
 #endif
 #ifdef BACKGROUND
@@ -1335,7 +1371,7 @@ static void __vga_mmap(void)
 				   PROT_READ | PROT_WRITE,
 				   MAP_SHARED,
 				   __svgalib_mem_fd,
-				   GRAPH_BASE
+				   __svgalib_graph_base
 	);
     __svgalib_map_virtual_screen(1000000); /* Video page */
 #endif	
@@ -1346,7 +1382,7 @@ static void __vga_mmap(void)
 				   PROT_READ | PROT_WRITE,
 				   MAP_SHARED,
 				   __svgalib_mem_fd,
-				   GRAPH_BASE
+				   __svgalib_graph_base
 	);
     __svgalib_graph_mem_orginal=GM;
 #endif
@@ -1380,7 +1416,7 @@ static void setcoloremulation(void)
     __svgalib_CRT_I = CRT_IC;
     __svgalib_CRT_D = CRT_DC;
     __svgalib_IS1_R = IS1_RC;
-    if (CHIPSET != EGA)
+    if (CHIPSET != EGA && !__svgalib_novga)  
 	port_out(port_in(MIS_R) | 0x01, MIS_W);
 }
 
@@ -1415,7 +1451,10 @@ static void initialize(void)
     graph_termio.c_oflag &= ~(ONOCR);
 
     graph_termio.c_lflag &= ~(ICANON | ECHO | ECHOE | ECHOK | ECHONL | NOFLSH);
-    graph_termio.c_lflag |= (ISIG);	/* enable interrupt */
+    if (__svgalib_nosigint)
+	graph_termio.c_lflag &= ~ISIG;	/* disable interrupt */
+    else
+	graph_termio.c_lflag |=  ISIG;	/* enable interrupt */
 
     graph_termio.c_cc[VMIN] = 1;
     graph_termio.c_cc[VTIME] = 0;
@@ -1427,7 +1466,8 @@ static void initialize(void)
     chipset_unlock();
 
     /* disable text output to console */
-    ioctl(__svgalib_tty_fd, KDSETMODE, KD_GRAPHICS);
+    if (!__svgalib_secondary)
+	ioctl(__svgalib_tty_fd, KDSETMODE, KD_GRAPHICS);
 
     __svgalib_takevtcontrol();	/* HH: Take control over VT */
 
@@ -1462,7 +1502,8 @@ static void initialize(void)
 	text_regs[EXT + 12] = port_in(__svgalib_CRT_D);
     }
     /* save text mode palette - first select palette index 0 */
-    port_out(0, PEL_IR);
+    if(!__svgalib_novga)
+	port_out(0, PEL_IR);
 
     /* read RGB components - index is autoincremented */
     savepalette(text_red, text_green, text_blue);
@@ -1471,48 +1512,52 @@ static void initialize(void)
     setcoloremulation();
 
     /* save font data - first select a 16 color graphics mode */
-    __svgalib_driverspecs->setmode(GPLANE16, prv_mode);
+    if (__svgalib_driverspecs->emul && __svgalib_driverspecs->emul->savefont) {
+	 __svgalib_driverspecs->emul->savefont();
+    } else if(!__svgalib_novga) {
+        __svgalib_driverspecs->setmode(GPLANE16, prv_mode);
 
-    /* Allocate space for textmode font. */
+        /* Allocate space for textmode font. */
 #ifndef BACKGROUND
-    font_buf1 = malloc(FONT_SIZE * 2);
+        font_buf1 = malloc(FONT_SIZE * 2);
 #endif
 #ifdef BACKGROUND
-    font_buf1 = valloc(FONT_SIZE * 2); 
+        font_buf1 = valloc(FONT_SIZE * 2); 
 #endif
-    font_buf2 = font_buf1 + FONT_SIZE;
+        font_buf2 = font_buf1 + FONT_SIZE;
 
     /* save font data in plane 2 */
-    port_out(0x04, GRA_I);
-    port_out(0x02, GRA_D);
+        port_out(0x04, GRA_I);
+        port_out(0x02, GRA_D);
 #ifdef __alpha__
-    port_out(0x06, GRA_I);
-    port_out(0x00, GRA_D);
+        port_out(0x06, GRA_I);
+        port_out(0x00, GRA_D);
 #endif
 #if defined(CONFIG_ALPHA_JENSEN)
-    slowcpy_from_sm(font_buf1, SM, FONT_SIZE);
+        slowcpy_from_sm(font_buf1, SM, FONT_SIZE);
 #else
-    slowcpy(font_buf1, GM, FONT_SIZE);
+        slowcpy(font_buf1, GM, FONT_SIZE);
 #endif
 
-    /* save font data in plane 3 */
-    port_out(0x04, GRA_I);
-    port_out(0x03, GRA_D);
+        /* save font data in plane 3 */
+        port_out(0x04, GRA_I);
+        port_out(0x03, GRA_D);
 #if defined(CONFIG_ALPHA_JENSEN)
-    slowcpy_from_sm(font_buf2, SM, FONT_SIZE);
+        slowcpy_from_sm(font_buf2, SM, FONT_SIZE);
 #else
-    slowcpy(font_buf2, GM, FONT_SIZE);
+        slowcpy(font_buf2, GM, FONT_SIZE);
 #endif
 
 #ifdef BACKGROUND
-    /* Let's protect font. */
-    /* Read only */
-    if (-1==mprotect(font_buf1,FONT_SIZE*2,PROT_READ))
-        {
-	 printf("svgalib: Memory protect error\n");
-	 exit(-1);
-	}
+        /* Let's protect font. */
+        /* Read only */
+        if (-1 == mprotect(font_buf1,FONT_SIZE*2,PROT_READ))
+            {
+	     printf("svgalib: Memory protect error\n");
+	     exit(-1);
+	    }
 #endif
+    }
     initialized = 1;
 
     /* do our own interrupt handling */
@@ -1714,19 +1759,22 @@ int vga_setmode(int mode)
     /* disable video */
     vga_screenoff();
 
+    if(!__svgalib_novga) {
     /* Should be more robust (eg. grabbed X modes) */
-    if (__svgalib_getchipset() == ET4000
-	&& prv_mode != G640x480x256
-	&& SVGAMODE(prv_mode))
-	chipset_setmode(G640x480x256, prv_mode);
+	if (__svgalib_getchipset() == ET4000
+	     && prv_mode != G640x480x256
+	     && SVGAMODE(prv_mode))
+	    chipset_setmode(G640x480x256, prv_mode);
 
-    /* This is a hack to get around the fact that some C&T chips
-     * are programmed to ignore syncronous resets. So if we are
-     * a C&T wait for retrace start */
-    if (__svgalib_getchipset() == CHIPS) {
-	while (((port_in(__svgalib_IS1_R)) & 0x08) == 0x08 );/* wait VSync off */
-	while (((port_in(__svgalib_IS1_R)) & 0x08) == 0 );   /* wait VSync on  */
-	port_outw(0x3C4,0x07);         /* reset hsync - just in case...  */
+	/* This is a hack to get around the fact that some C&T chips
+	 * are programmed to ignore syncronous resets. So if we are
+	 * a C&T wait for retrace start
+	 */
+	if (__svgalib_getchipset() == CHIPS) {
+	   while (((port_in(__svgalib_IS1_R)) & 0x08) == 0x08 );/* wait VSync off */
+	   while (((port_in(__svgalib_IS1_R)) & 0x08) == 0 );   /* wait VSync on  */
+	   port_outw(0x3C4,0x07);         /* reset hsync - just in case...  */
+	}
     }
 
     if (mode == TEXT) {
@@ -1750,65 +1798,71 @@ int vga_setmode(int mode)
 	/* restore font data - first select a 16 color graphics mode */
 	/* Note: this should restore the old extended registers if */
 	/* setregs is not defined for the chipset. */
-	__svgalib_driverspecs->setmode(GPLANE16, prv_mode);
+        if(__svgalib_novga) __svgalib_driverspecs->setmode(TEXT, prv_mode);
+        if (__svgalib_driverspecs->emul && __svgalib_driverspecs->emul->restorefont) {
+           __svgalib_driverspecs->emul->restorefont(); 
+	   chipset_setregs(text_regs, mode);
+        } else if(!__svgalib_novga) {
+	  __svgalib_driverspecs->setmode(GPLANE16, prv_mode);
 
-	if (CHIPSET != EGA)
-	    /* restore old extended regs */
-	    chipset_setregs(text_regs, mode);
+	  if (CHIPSET != EGA)
+	      /* restore old extended regs */
+	      chipset_setregs(text_regs, mode);
 
-	/* disable Set/Reset Register */
-	port_out(0x01, GRA_I);
-	port_out(0x00, GRA_D);
+	  /* disable Set/Reset Register */
+	  port_out(0x01, GRA_I);
+	  port_out(0x00, GRA_D);
 
-	prepareforfontloading();
+	  prepareforfontloading();
 
-	/* restore font data in plane 2 - necessary for all VGA's */
-	port_out(0x02, SEQ_I);
-	port_out(0x04, SEQ_D);
+	  /* restore font data in plane 2 - necessary for all VGA's */
+	  port_out(0x02, SEQ_I);
+	  port_out(0x04, SEQ_D);
 #ifdef __alpha__
-	port_out(0x06, GRA_I);
-	port_out(0x00, GRA_D);
+	  port_out(0x06, GRA_I);
+	  port_out(0x00, GRA_D);
 #endif
 
 #ifdef BACKGROUND
-        if (-1==mprotect(font_buf1,FONT_SIZE*2,PROT_READ|PROT_WRITE))
-        {
-	 printf("svgalib: Memory protect error\n");
-	 exit(-1);
-	}
+          if (-1 == mprotect(font_buf1,FONT_SIZE*2,PROT_READ|PROT_WRITE))
+          {
+	   printf("svgalib: Memory protect error\n");
+	   exit(-1);
+	  }
 #endif
 
 #if defined(CONFIG_ALPHA_JENSEN)
-	slowcpy_to_sm(SM, font_buf1, FONT_SIZE);
+	  slowcpy_to_sm(SM, font_buf1, FONT_SIZE);
 #else
-	slowcpy(GM, font_buf1, FONT_SIZE);
+	  slowcpy(GM, font_buf1, FONT_SIZE);
 #endif
 
-	/* restore font data in plane 3 - necessary for Trident VGA's */
-	port_out(0x02, SEQ_I);
-	port_out(0x08, SEQ_D);
+	  /* restore font data in plane 3 - necessary for Trident VGA's */
+	  port_out(0x02, SEQ_I);
+	  port_out(0x08, SEQ_D);
 #if defined(CONFIG_ALPHA_JENSEN)
-	slowcpy_to_sm(SM, font_buf2, FONT_SIZE);
+	  slowcpy_to_sm(SM, font_buf2, FONT_SIZE);
 #else
-	slowcpy(GM, font_buf2, FONT_SIZE);
+	  slowcpy(GM, font_buf2, FONT_SIZE);
 #endif
 #ifdef BACKGROUND
-        if (-1==mprotect(font_buf1,FONT_SIZE*2,PROT_READ))
-        {
-	 printf("svgalib: Memory protect error\n");
-	 exit(1);
-	}
+          if (-1 == mprotect(font_buf1,FONT_SIZE*2,PROT_READ))
+          {
+	   printf("svgalib: Memory protect error\n");
+	   exit(1);
+	  }
 #endif
-	fontloadingcomplete();
+	  fontloadingcomplete();
 
-	/* change register adresses if monochrome text mode */
-	/* EGA is assumed to use color emulation. */
-	if (!color_text) {
-	    __svgalib_CRT_I = CRT_IM;
-	    __svgalib_CRT_D = CRT_DM;
-	    __svgalib_IS1_R = IS1_RM;
-	    port_out(port_in(MIS_R) & 0xFE, MIS_W);
-	}
+	  /* change register adresses if monochrome text mode */
+	  /* EGA is assumed to use color emulation. */
+	  if (!color_text) {
+	      __svgalib_CRT_I = CRT_IM;
+	      __svgalib_CRT_D = CRT_DM;
+	      __svgalib_IS1_R = IS1_RM;
+	      port_out(port_in(MIS_R) & 0xFE, MIS_W);
+	  }
+        } else chipset_setregs(text_regs, mode);
 	/* restore saved palette */
 	restorepalette(text_red, text_green, text_blue);
 
@@ -1822,7 +1876,8 @@ int vga_setmode(int mode)
 
 /*      if (!flip) */
 	/* enable text output - restores the screen contents */
-	ioctl(__svgalib_tty_fd, KDSETMODE, KD_TEXT);
+        if (!__svgalib_secondary)
+	    ioctl(__svgalib_tty_fd, KDSETMODE, KD_TEXT);
 
 	usleep(MODESWITCHDELAY);	/* wait for signal to stabilize */
 
@@ -1836,7 +1891,8 @@ int vga_setmode(int mode)
 	/* Setting a graphics mode. */
 
 	/* disable text output */
-	ioctl(__svgalib_tty_fd, KDSETMODE, KD_GRAPHICS);
+        if (!__svgalib_secondary)
+	    ioctl(__svgalib_tty_fd, KDSETMODE, KD_GRAPHICS);
 
 	if (SVGAMODE(prv_mode)) {
 	    /* The current mode is an SVGA mode, and we now want to */
@@ -1871,6 +1927,7 @@ int vga_setmode(int mode)
 
 	    modeinfo = vga_getmodeinfo(mode);
 	    VMEM = modeinfo->linewidth * modeinfo->height;
+            CI.xbytes = modeinfo->linewidth;
 	}
 
 	if (!flip) {
@@ -1946,7 +2003,7 @@ void vga_puttextfont(void *font)
     unsigned int putsize;
 
 #ifdef BACKGROUND
-        if (-1==mprotect(font_buf1,FONT_SIZE*2,PROT_READ|PROT_WRITE))
+        if (-1 == mprotect(font_buf1,FONT_SIZE*2,PROT_READ|PROT_WRITE))
         {
 	 printf("svgalib: Memory protect error\n");
 	 exit(-1);
@@ -1964,7 +2021,7 @@ void vga_puttextfont(void *font)
     }
 
 #ifdef BACKGROUND
-        if (-1==mprotect(font_buf1,FONT_SIZE*2,PROT_READ))
+        if (-1 == mprotect(font_buf1,FONT_SIZE*2,PROT_READ))
         {
 	 printf("svgalib: Memory protect error\n");
 	 exit(-1);
@@ -2002,9 +2059,10 @@ vga_modeinfo *vga_getmodeinfo(int mode)
     static vga_modeinfo modeinfo;
     int is_modeX = (CM == mode) && MODEX;
 
+    modeinfo.linewidth = infotable[mode].xbytes;
+    __svgalib_getchipset();
     if (mode > vga_lastmodenumber())
 	return NULL;
-    __svgalib_getchipset();
     modeinfo.width = infotable[mode].xdim;
     modeinfo.height = infotable[mode].ydim;
     modeinfo.bytesperpixel = infotable[mode].bytesperpixel;
@@ -2012,8 +2070,7 @@ vga_modeinfo *vga_getmodeinfo(int mode)
     if (is_modeX) {
 	modeinfo.linewidth = modeinfo.width / 4;
 	modeinfo.bytesperpixel = 0;
-    } else
-	modeinfo.linewidth = infotable[mode].xbytes;
+    }
     if (mode == TEXT) {
 	modeinfo.flags = HAVE_EXT_SET;
 	return &modeinfo;
@@ -2123,76 +2180,71 @@ int vga_setcolor(int color)
 
 int vga_screenoff(void)
 {
-#ifdef BACKGROUND 
- __svgalib_dont_switch_vt_yet();
- if (!__svgalib_oktowrite)
-     {
-      SCREENON=0;
-      __svgalib_is_vt_switching_needed();
-      return(0);
-     }
-#endif
-    /* Dunno how to support that in MACH32 modes... */
-    /* But it is used to switch the VGA off for better
-       mem access performance... */
-    /* turn off screen for faster VGA memory acces */
-    if (CHIPSET != EGA) {
-	port_out(0x01, SEQ_I);
-	port_out(port_in(SEQ_D) | 0x20, SEQ_D);
-    }
-    /* Disable video output */
-#ifdef DISABLE_VIDEO_OUTPUT
-    port_in(__svgalib_IS1_R);
-    __svgalib_delay();
-    port_out(0x00, ATT_IW);
-#endif
+    int tmp = 0;
 
     SCREENON = 0;
+
+    if(__svgalib_novga) return 0; 
+#ifdef BACKGROUND 
+    __svgalib_dont_switch_vt_yet();
+    if (!__svgalib_oktowrite) {
+	__svgalib_is_vt_switching_needed();
+	return(0);
+    }
+#endif
+
+    if (__svgalib_driverspecs->emul && __svgalib_driverspecs->emul->screenoff) {
+	tmp = __svgalib_driverspecs->emul->screenoff();
+    } else {
+	/* turn off screen for faster VGA memory acces */
+	if (CHIPSET != EGA) {
+	    port_out(0x01, SEQ_I);
+	    port_out(port_in(SEQ_D) | 0x20, SEQ_D);
+	}
+	/* Disable video output */
+#ifdef DISABLE_VIDEO_OUTPUT
+	port_in(__svgalib_IS1_R);
+	__svgalib_delay();
+	port_out(0x00, ATT_IW);
+#endif
+    }
+
 #ifdef BACKGROUND
     __svgalib_is_vt_switching_needed();
 #endif
-    return 0;
+    return tmp;
 }
 
 
 int vga_screenon(void)
 {
-#ifdef BACKGROUND
- __svgalib_dont_switch_vt_yet();
- if (!__svgalib_oktowrite)
-     {
-      SCREENON=1;
-      __svgalib_is_vt_switching_needed();
-      return(0);
-     }
-#endif
-    /* Dunno how to support that in MACH32 modes... */
-    if ((__svgalib_chipset == MACH32) && SVGAMODE(CM)) {
-	/* Warning! This is also used for faking of vgapal.c! */
-	/* Not needed anymore... had to hack vgapal.c anyway.. */
-	/* Let it in anyway to show that std-vga is disabled */
-
-	/* Anyway force Mach32 to ATI mode (used by setmode) */
-	outw(0x4AEE, inw(0x4AEE) | 1);
-	SCREENON = 0;
-#ifdef BACKGROUND
-        __svgalib_is_vt_switching_needed();
-#endif 
-	return 0;
-    }
-    /* turn screen back on */
-    if (CHIPSET != EGA) {
-	port_out(0x01, SEQ_I);
-	port_out(port_in(SEQ_D) & 0xDF, SEQ_D);
-    }
-/* #ifdef DISABLE_VIDEO_OUTPUT */
-    /* enable video output */
-    port_in(__svgalib_IS1_R);
-    __svgalib_delay();
-    port_out(0x20, ATT_IW);
-/* #endif */
+    int tmp = 0;
 
     SCREENON = 1;
+    if(__svgalib_novga) return 0; 
+#ifdef BACKGROUND
+    __svgalib_dont_switch_vt_yet();
+    if (!__svgalib_oktowrite) {
+	__svgalib_is_vt_switching_needed();
+	return(0);
+    }
+#endif
+    if (__svgalib_driverspecs->emul && __svgalib_driverspecs->emul->screenon) {
+	tmp = __svgalib_driverspecs->emul->screenon();
+    } else {
+	/* turn screen back on */
+	if (CHIPSET != EGA) {
+	    port_out(0x01, SEQ_I);
+	    port_out(port_in(SEQ_D) & 0xDF, SEQ_D);
+	}
+/* #ifdef DISABLE_VIDEO_OUTPUT */
+	/* enable video output */
+	port_in(__svgalib_IS1_R);
+	__svgalib_delay();
+	port_out(0x20, ATT_IW);
+/* #endif */
+    }
+
 #ifdef BACKGROUND
     __svgalib_is_vt_switching_needed();
 #endif
@@ -2564,6 +2616,9 @@ void __svgalib_flipaway(void)
 {
     /* Leaving console. */
     flip_mode = CM;
+#ifndef SVGA_AOUT
+    __joystick_flip_vc(0);
+#endif
     if (CM != TEXT) {
 	/* wait for any blitter operation to finish */
 	idle_accel();
@@ -2600,6 +2655,9 @@ void __svgalib_flipback(void)
 #endif
     /* Entering console. */
     /* Hmmm... and how about unlocking anything someone else locked? */
+#ifndef SVGA_AOUT
+    __joystick_flip_vc(1);
+#endif
     chipset_unlock();
     if (flip_mode != TEXT) {
 	/* Restore graphics mode and state. */
@@ -2849,71 +2907,119 @@ static void parse_string(char *str, char **commands, char *(*func) (int ind, int
     }
 }
 
-static int allowoverride = 0;	/* Allow dangerous options in ENV-Var */
+static int allowoverride = 0;	/* Allow dangerous options in ENV-Var or in */
+				/* the $HOME/.svgalibrc */
 
-/* This is a service function for drivers. commands and func are as above,
-   the config file and (afterwards) the contents of the environment variable
-   SVGALIB_CONFIG are parsed.. */
-void __svgalib_read_options(char **commands, char *(*func) (int ind, int mode))
-{
-/* Parse configuration file. */
-    FILE *f;
+static void process_config_file(FILE *file, int mode, char **commands,
+		 			char *(*func)(int ind, int mode)) {
+ struct stat st;
+ char *buf, *ptr;
+ int i;
+
+  fstat(fileno(file), &st);	/* Some error analysis may be fine here.. */
+  if ( (buf = alloca(st.st_size + 1)) == 0) {	/* + a final \0 */
+    puts("svgalib: out of mem while parsing config file !");
+    return;
+  }
+  fread(buf, 1, st.st_size, file);
+  for (i = 0, ptr = buf; i < st.st_size; i++, ptr++) {
+    if (!*ptr)
+      *ptr = ' ';			/* Erase any maybe embedded \0 */
+    }
+  *ptr = 0;					/* Trailing \0 */
+  parse_string(buf, commands, func, mode);	/* parse config file */
+}
+
+/* This is a service function for drivers. Commands and func are as above.
+   The following config files are parsed in this order:
+    - /etc/vga/libvga.conf (#define SVGALIB_CONFIG_FILE)
+    - ~/.svgalibrc
+    - the file where env variavle SVGALIB_CONFIG_FILE points
+    - the env variable SVGALIB_CONFIG (for compatibility, but I would remove
+      it, we should be more flexible...  Opinions ?)
+    - MW: I'd rather keep it, doesn't do too much harm and is sometimes nice
+      to have.
+*/
+void __svgalib_read_options(char **commands, char *(*func) (int ind, int mode)) {
+    FILE *file;
     char *buf = NULL, *ptr;
-    struct stat st;
     int i;
 
-    st.st_size = 0;		/*safety for readenv */
+    if ( (file = fopen(SVGALIB_CONFIG_FILE, "r")) != 0) {
+#ifdef DEBUG_CONF
+  printf("Processing config file \'%s\'\n", SVGALIB_CONFIG_FILE);
+#endif
+      process_config_file(file, 1, commands, func);
+      fclose(file);
+    } else {
+	fprintf(stderr, "svgalib: Configuration file \'%s\' not found.\n", SVGALIB_CONFIG_FILE);
+    }
 
-    f = fopen(SVGALIB_CONFIG_FILE, "r");
-    if (f == NULL) {
-	printf("svgalib: Configuration file %s not found.\n",
-	       SVGALIB_CONFIG_FILE);
-	goto readenv;
+    if ( (ptr = getenv("HOME")) != 0) {
+      char *filename; 
+
+      filename = alloca(strlen(ptr) + 20);
+      if (!filename) {
+	puts("svgalib: out of mem while parsing SVGALIB_CONFIG_FILE !");
+      } else {
+	strcpy(filename, ptr);
+	strcat(filename, "/.svgalibrc");
+	if ( (file = fopen(filename, "r")) != 0) {
+#ifdef DEBUG_CONF
+	  printf("Processing config file \'%s\'\n", filename);
+#endif
+	  process_config_file(file, allowoverride, commands, func);
+	  fclose(file);
+	}
+      }
     }
-    fstat(fileno(f), &st);	/* Some error analysis may be fine here.. */
-    buf = alloca(st.st_size + 1);	/* + a final \0 */
-    fread(buf, 1, st.st_size, f);
-    fclose(f);
-    /*Erase any maybe embedded \0 */
-    for (i = 0, ptr = buf; i < st.st_size; i++, ptr++) {
-	if (!*ptr)
-	    *ptr = ' ';
+
+    if ( (ptr = getenv("SVGALIB_CONFIG_FILE")) != 0) {
+      if ( (file = fopen(ptr, "r")) != 0) {
+#ifdef DEBUG_CONF
+  printf("Processing config file \'%s\'\n", ptr);
+#endif
+	process_config_file(file, allowoverride, commands, func);
+	fclose(file);
+      } else {
+        fprintf(stderr, "svgalib: warning: config file \'%s\', pointed to by SVGALIB_CONFIG_FILE, not found !\n", ptr);
+      }
+    }
+
+    if ( (ptr = getenv("SVGALIB_CONFIG")) != 0  &&  (i = strlen(ptr)) != 0) {
+      buf = alloca(i + 1);
+      if (!buf) {
+	puts("svgalib: out of mem while parsing SVGALIB_CONFIG !");
+      } else {
+	strcpy(buf, ptr);		/* Copy for safety and strtok!! */
+#ifdef DEBUG_CONF
+	puts("Parsing env variable \'SVGALIB_CONFIG\'");
+#endif
+	parse_string(buf, commands, func, allowoverride);
+      }
     }
-    /*Trailing \0 */
-    *ptr = 0;
-    /*parse config file */
-    parse_string(buf, commands, func, 1);
-  readenv:
-    if (!(ptr = getenv("SVGALIB_CONFIG")))
-	return;
-    if (!(i = strlen(ptr)))
-	return;
-    if (i > st.st_size)		/* Note that we allocated one more char... */
-	buf = alloca(i + 1);
-    strcpy(buf, ptr);		/* Copy for safety and strtok!! */
-    /*parse environment variable */
-    parse_string(buf, commands, func, allowoverride);
 }
 
 /* Configuration file, mouse interface, initialization. */
 
 static int configfileread = 0;	/* Boolean. */
 
-static char *vga_conf_commands[] =
-{
+/* What are these m0 m1 m... things ? Shouldn't they be removed ? */
+static char *vga_conf_commands[] = {
     "mouse", "monitor", "!m", "!M", "chipset", "overrideenable", "!m0", "!m1", "!m2", "!m3",
-"!m4", "!m9", "!M0", "!M1", "!M2", "!M3", "!M4", "!M5", "!M6", "nolinear",
+    "!m4", "!m9", "!M0", "!M1", "!M2", "!M3", "!M4", "!M5", "!M6", "nolinear",
     "linear", "!C0", "!C1", "!C2", "!C3", "!C4", "!C5", "!C6", "!C7", "!C8", "!C9",
     "!c0", "!c1", "monotext", "colortext", "!m5",
     "leavedtr", "cleardtr", "setdtr", "leaverts", "clearrts",
     "setrts", "grayscale", "horizsync", "vertrefresh", "modeline",
-    "security","mdev",
+    "security","mdev", "default_mode", "nosigint", "sigint",
+    "joystick0", "joystick1", "joystick2", "joystick3",
     NULL};
 
 static char *conf_mousenames[] =
 {
   "Microsoft", "MouseSystems", "MMSeries", "Logitech", "Busmouse", "PS2",
-    "MouseMan", "gpm", "Spaceball", NULL};
+    "MouseMan", "gpm", "Spaceball", "none", "IntelliMouse", "IMPS2", NULL};
 
 static int check_digit(char *ptr, char *digits)
 {
@@ -2949,14 +3055,10 @@ static char *process_option(int command, int mode)
 	    goto inv_mouse;
 	if (check_digit(ptr, digits + 1)) {	/* It is a number.. */
 	    i = atoi(ptr);
-	    if ((i != 9) && (i >= sizeof(conf_mousenames)/sizeof(char *)))
+	    if ((i < 0) || (i > 9))
 		goto inv_mouse;
 	    mouse_type = i;
 	} else {		/* parse for symbolic name.. */
-	    if (!strcasecmp(ptr, "none")) {
-		mouse_type = 9;
-		break;
-	    }
 	    for (i = 0, tabptr = conf_mousenames; *tabptr; tabptr++, i++) {
 		if (!strcasecmp(ptr, *tabptr)) {
 		    mouse_type = i;
@@ -2968,7 +3070,9 @@ static char *process_option(int command, int mode)
 		   "Correct usage: {mouse|m} mousetype\n"
 		   "where mousetype is one of 0, 1, 2, 3, 4, 5, 6, 7, 9,\n",
 		   (ptr != NULL) ? ptr : "");
-	    for (tabptr = conf_mousenames; *tabptr; tabptr++) {
+	    for (tabptr = conf_mousenames, i = 0; *tabptr; tabptr++, i++) {
+		if (i == MOUSE_NONE)
+		    continue;
 		printf("%s, ", *tabptr);
 	    }
 	    puts("or none.");
@@ -3064,7 +3168,7 @@ static char *process_option(int command, int mode)
 	mouse_type = command - 6;
 	break;
     case 11:			/* m9 */
-	mouse_type = 9;
+	mouse_type = MOUSE_NONE;
 	break;
     case 12:			/* oldstyle config: M0-M6 */
     case 13:
@@ -3283,11 +3387,52 @@ static char *process_option(int command, int mode)
 	if (ptr) {
 	    mouse_device = strdup(ptr);
 	    if (mouse_device == NULL) {
+#ifndef SVGA_AOUT
+	      nomem:
+#endif
 		puts("svgalib: Fatal error: out of memory.");
 		exit(1);
 	    }
 	} else
-	    puts("libvga.config: mdev requires an argument.");
+	    goto param_needed;
+	break;
+    case 48:		/* default_mode */
+	if ( (ptr = strtok(NULL, " ")) != 0) {
+	 int mode = vga_getmodenumber(ptr);
+	  if (mode != -1) {
+	    __svgalib_default_mode = mode;
+	  } else {
+	    printf("svgalib: config: illegal mode \'%s\' for \'%s\'\n",
+	   			  ptr, vga_conf_commands[command]);
+	  }
+	} else {
+  param_needed:
+  	  printf("svgalib: config: \'%s\' requires parameter(s)",
+  	  				vga_conf_commands[command]);
+	  break;
+	}
+	break;
+    case 49: /* nosigint */
+	__svgalib_nosigint = 1;
+	break;
+    case 50: /* sigint */
+	__svgalib_nosigint = 0;
+	break;
+    case 51: /* joystick0 */
+    case 52: /* joystick1 */
+    case 53: /* joystick2 */
+    case 54: /* joystick3 */
+	if (! (ptr = strtok(NULL, " ")) )
+		goto param_needed;
+#ifndef SVGA_AOUT
+	if (__joystick_devicenames[command - 51])
+	    free(__joystick_devicenames[command - 51]);
+	__joystick_devicenames[command - 51] = strdup(ptr);
+	if (!__joystick_devicenames[command - 51])
+	    goto nomem;
+#else
+	printf("svgalib: No joystick support in a.out version.\n");
+#endif
 	break;
     }
   leave:
