@@ -26,26 +26,24 @@
 #include "vgabg.h"
 #endif
 
-/* This routine is a pain in my ass and definitely needs to be virtualized..
-   M. Weller. (mach32 support) */
 void vga_waitretrace(void)
 {
-    if ((__svgalib_chipset == MACH32) && SVGAMODE(CM)) {
-	unsigned char csync;
-
-	csync = inb(0xd2ee) >> 3;	/* Current sync polarity in bit 2,
-					   0-pos, 1-neg  */
-	/*Wait until in displayed area.. */
-	while ((inb(0x2e8) ^ csync) & 2);
-	/*Wait until V_SYNC */
-	csync = ~csync;
-	while ((inb(0x2e8) ^ csync) & 2);
-	/* Hmm, is a return missing here? -- HH */
-	/* Umm.. *scratch head* Certainly.. -- MW (mach32) */
+#ifdef BACKGROUND
+    __svgalib_dont_switch_vt_yet();
+    if (!vga_oktowrite()) {
+	__svgalib_is_vt_switching_needed();
 	return;
     }
-    while (!(inb(0x3da) & 8));
-    while (inb(0x3da) & 8);
+#endif
+    if (__svgalib_driverspecs->emul && __svgalib_driverspecs->emul->waitretrace) {
+        __svgalib_driverspecs->emul->waitretrace();
+    } else {
+	while (!(inb(0x3da) & 8));
+	while (inb(0x3da) & 8);
+    }
+#ifdef BACKGROUND
+    __svgalib_is_vt_switching_needed();
+#endif
 }
 
 #ifndef BACKGROUND
