@@ -1,9 +1,9 @@
 /*
 
-    3DKIT   version   1.2
+    3DKIT   version   1.3
     High speed 3D graphics and rendering library for Linux.
 
-    Copyright (C) 1996  Paul Sheer   psheer@hertz.mech.wits.ac.za
+    Copyright (C) 1996, 1997  Paul Sheer   psheer@icon.co.za
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -22,6 +22,7 @@
 
 */
 
+
 /*
 
 File: 3dkit.c
@@ -74,29 +75,30 @@ centre, or as a 3D world. (See plane.h for how to modify the demo).
 */
 
 
-
-
-
-
 #define TD_MULCONSTANT 4096
 
+#include <config.h>
 #include <math.h>
 #include <stdlib.h>
 #include <stdio.h>
-#include <vga.h>
 #include <vgagl.h>
 #include "3dkit.h"
 
+#define max(x,y)     (((x) > (y)) ? (x) : (y))
+#define min(x,y)     (((x) < (y)) ? (x) : (y))
+
 
 /*global for holding a surface temporarily:*/
 TD_Short_Point *temp;
 
+
+#if defined(__GNUC__) && !defined(__STRICT_ANSI__)
+
 /* The optimisation comes from svgalib-1.2.9/gl/line.c: */
 
 /* Framebuffer Graphics Libary for Linux, Copyright 1993 Harm Hanemaayer */
 /* line.c       Line drawing */
 
-#ifdef __linux__
 #ifdef __alpha__
 
 static inline int muldiv64 (int m1, int m2, int d)
@@ -106,6 +108,8 @@ static inline int muldiv64 (int m1, int m2, int d)
 
 #else
 
+#ifdef __i386__
+
 /* We use the 32-bit to 64-bit multiply and 64-bit to 32-bit divide of the */
 /* 386 (which gcc doesn't know well enough) to efficiently perform integer */
 /* scaling without having to worry about overflows. */
@@ -124,82 +128,89 @@ static inline int muldiv64 (int m1, int m2, int d)
     return result;
 }
 
-#endif				/* !__alpha__ */
-#endif				/* __linux__ */
+#else
 
+static inline int muldiv64(int m1, int m2, int d)
+{
+    return (double) m1 * (double) m2 / ((double) d);
+}
+
+#endif				/* !__i386__ */
+#endif				/* !__alpha__ */
+
+#else
+
+#define muldiv64(a,b,c) ((long) ((double) a * (double) b / ((double) c)))
+
+#endif
 
 void TD_translate (TD_Solid * s, TD_Point * p, TD_Short_Point * scr)
 {
 /* the following rotational transformation avoids floating point
    calculations entirely */
 
-    long xt, yt, zt;
-    long x, y, z;
     if (s->option_flags & TDOPTION_32BIT_SURFACES) {
-	x = (p->x + s->x_cam) >> 16;
-	y = (p->y + s->y_cam) >> 16;
-	z = (p->z + s->z_cam) >> 16;
+/* for super accuracy */
+	double x = p->x + s->x_cam;
+	double y = p->y + s->y_cam;
+	double z = p->z + s->z_cam;
+	double yt = x * s->a21 + y * s->a22 + z * s->a23 + s->s_cam;
+
+	if (yt < 1) {
+	    scr->x = scr->y = 32767;
+	    return;
+	} else {
+	    double xt = x * s->a11 + y * s->a12 + z * s->a13;
+	    double zt = x * s->a31 + y * s->a32 + z * s->a33;
+	    scr->x = ((long) ((double) s->posx + xt * s->xscale / yt)) >> 16;
+	    scr->y = ((long) ((double) s->posy - zt * s->yscale / yt)) >> 16;
+	    return;
+	}
     } else {
-	x = p->x + s->x_cam;
-	y = p->y + s->y_cam;
-	z = p->z + s->z_cam;
-    }
-
-#ifndef __linux__
-
-/*To avoid overflow with 32 bit longs, we count the bits:*/
+	long x = p->x + s->x_cam;
+	long y = p->y + s->y_cam;
+	long z = p->z + s->z_cam;
+	long yt = x * s->a21 + y * s->a22 + z * s->a23 + s->s_cam;
 
 /*(FIXME:) There may be problems if yt overflows, this just checks if the point
- is behind the cam:*/
-    if((xt = (x * s->a11 + y * s->a12 + z * s->a13 + s->s_cam) >> 16) < 1) {
-	scr->x = scr->y = 32767; /*line and triangle routines must 
-				    reject these values.*/
-	return;
+   is behind the cam: */
+	if (yt < 1) {
+	    scr->x = scr->y = 32767;	/*line and triangle routines must 
+					   reject these values. */
+	    return;
+	} else {
+	    long xt = x * s->a11 + y * s->a12 + z * s->a13;
+	    long zt = x * s->a31 + y * s->a32 + z * s->a33;
+	    scr->x = s->posx + muldiv64 (xt, s->xscale, yt);
+	    scr->y = s->posy - muldiv64 (zt, s->yscale, yt);
+	    return;
+	}
     }
-    yt = (x * s->a21 + y * s->a22 + z * s->a23) >> 16;
-    zt = (x * s->a31 + y * s->a32 + z * s->a33) >> 16;
-
-    scr->x = (long) s->posx + yt * s->xscale / xt;
-    scr->y = (long) s->posy - zt * s->yscale / xt;
-
-#else
-/*(FIXME:) There may be problems if yt overflows, this just checks if the point
- is behind the cam:*/
-    if((xt = x * s->a11 + y * s->a12 + z * s->a13 + s->s_cam) < 1) {
-	scr->x = scr->y = 32767; /*line and triangle routines must 
-				    reject these values.*/
-	return;
-    }
-    yt = x * s->a21 + y * s->a22 + z * s->a23;
-    zt = x * s->a31 + y * s->a32 + z * s->a33;
-
-    scr->x = s->posx + muldiv64 (yt, s->xscale, xt);
-    scr->y = s->posy - muldiv64 (zt, s->yscale, xt);
-
-#endif
-
 }
 
-/*--> I wonder if those were worth the effort.*/
 
 
 long TD_finddistance (TD_Solid * s, TD_Point * p)
 {
-    long x, y, z;
-    if(s->option_flags & TDOPTION_32BIT_SURFACES) {
-	x = p->x>>16;
-	y = p->y>>16;
-	z = p->z>>16;
-    } else {
-	x = p->x;
-	y = p->y;
-	z = p->z;
-    }
+/* the following rotational transformation avoids floating point
+   calculations entirely */
 
-    return (x * s->a11 + y * s->a12 + z * s->a13 + s->s_cam) >> 16;
+    if (s->option_flags & TDOPTION_32BIT_SURFACES) {
+/* for super accuracy */
+	double x = p->x + s->x_cam;
+	double y = p->y + s->y_cam;
+	double z = p->z + s->z_cam;
+	return ((long) ((double) x * s->a21 + y * s->a22 + z * s->a23 + s->s_cam)) >> 16;
+    } else {
+	long x = p->x + s->x_cam;
+	long y = p->y + s->y_cam;
+	long z = p->z + s->z_cam;
+	return (x * s->a21 + y * s->a22 + z * s->a23 + s->s_cam);
+    }
 }
 
 
+
 long TD_findcolor (TD_Solid * s, TD_Point * p, int which)
 {
     long c, shadow = s->surf[which].shadow;
@@ -245,17 +256,34 @@ void TD_calc_rotation_matrix (TD_Solid * s)
    John Wiley & Sons, Inc., and is much easier to copy down than to 
    derive yourself. */
 
-    s->a11 = (float) TD_MULCONSTANT * cos (s->alpha) * cos (s->gamma);
-    s->a12 = (float) TD_MULCONSTANT * cos (s->alpha) * sin (s->gamma);
-    s->a13 = (float) TD_MULCONSTANT * -sin (s->alpha);
+    float tsi = s->alpha, theta = s->beta, phi = s->gamma;
+
+    s->a22 = (float) TD_MULCONSTANT * (cos (theta) * cos (tsi));
+    s->a21 = (float) TD_MULCONSTANT * (cos (theta) * sin (tsi));
+    s->a23 = (float) TD_MULCONSTANT * (-sin (theta));
+
+    s->a12 = (float) TD_MULCONSTANT * (sin (phi) * sin (theta) * cos (tsi) - cos (phi) * sin (tsi));
+    s->a11 = (float) TD_MULCONSTANT * (sin (phi) * sin (theta) * sin (tsi) + cos (phi) * cos (tsi));
+    s->a13 = (float) TD_MULCONSTANT * (sin (phi) * cos (theta));
+
+    s->a32 = (float) TD_MULCONSTANT * (cos (phi) * sin (theta) * cos (tsi) + sin (phi) * sin (tsi));
+    s->a31 = (float) TD_MULCONSTANT * (cos (phi) * sin (theta) * sin (tsi) - sin (phi) * cos (tsi));
+    s->a33 = (float) TD_MULCONSTANT * (cos (phi) * cos (theta));
+
+/* this is the classical rotations matrix of aerodynamics */
+/*
+    s->a11 = (float) TD_MULCONSTANT * (cos (s->alpha) * cos (s->gamma));
+    s->a12 = (float) TD_MULCONSTANT * (cos (s->alpha) * sin (s->gamma));
+    s->a13 = (float) TD_MULCONSTANT * (-sin (s->alpha));
 
     s->a21 = (float) TD_MULCONSTANT * (sin (s->beta) * sin (s->alpha) * cos (s->gamma) - cos (s->beta) * sin (s->gamma));
-    s->a22 = (float) TD_MULCONSTANT * (sin (s->beta) * sin (s->alpha) * sin (s->gamma) + cos (s->beta) * cos (s->gamma));
-    s->a23 = (float) TD_MULCONSTANT * sin (s->beta) * cos (s->alpha);
+    s->a22 = (float) TD_MULCONSTANT * (sin (s->beta) * sin (s->alpha) * sin (s->gamma) - cos (s->beta) * cos (s->gamma));
+    s->a23 = (float) TD_MULCONSTANT * (sin (s->beta) * cos (s->alpha));
 
     s->a31 = (float) TD_MULCONSTANT * (cos (s->beta) * sin (s->alpha) * cos (s->gamma) + sin (s->beta) * sin (s->gamma));
-    s->a32 = (float) TD_MULCONSTANT * (cos (s->beta) * sin (s->alpha) * sin (s->gamma) - sin (s->beta) * cos (s->gamma));
-    s->a33 = (float) TD_MULCONSTANT * cos (s->beta) * cos (s->alpha);
+    s->a32 = (float) TD_MULCONSTANT * (cos (s->beta) * sin (s->alpha) * sin (s->gamma) + sin (s->beta) * cos (s->gamma));
+    s->a33 = (float) TD_MULCONSTANT * (cos (s->beta) * cos (s->alpha));
+*/
 
 /*results are 14 bit + sign integers*/
 }
@@ -285,9 +313,10 @@ void TD_drawwire (TD_Solid * s, int which)
 	j -= w;
     }
 
-    for (j = 0; j < i - 1; j++)
+    for (j = 0; j < i - 1; j++) {
 	(*dl) (temp[j].x, temp[j].y, temp[j + 1].x, temp[j + 1].y, c);
 
+    }
 }
 
 
@@ -310,6 +339,7 @@ void TD_drawmesh (TD_Solid * s, int which)
 	    (*dl) (temp[k + w].x, temp[k + w].y, temp[k].x, temp[k].y, c);
 	}
 	(*dl) (temp[k + w].x, temp[k + w].y, temp[k].x, temp[k].y, c);
+
     }
 
     for (i = 0; i < w - 1; i++, k++)
@@ -318,43 +348,6 @@ void TD_drawmesh (TD_Solid * s, int which)
 }
 
 
-/*
-drawtoolpath(struct identitiestype *ident)
-{
-int tx,ty,tz;
-int j;
-int px2,py2,px3,py3;
-struct ptemptype scr;
-if(nocnc==1) return(0);
-
-        tx=(*toolpoints)[0].x / maxcoord * MULSHIFT;
-        ty=(*toolpoints)[0].y / maxcoord * MULSHIFT;
-        tz=(*toolpoints)[0].z / maxcoord * MULSHIFT;
-        
-		translate(tx,ty,tz,ident,&scr);
-
-        px2=scr.x;
-        py2=scr.y;
-        
-        if(nmoves!=0) for(j=1;j<nmoves;j++)
-        {
-        px3=px2;
-        py3=py2;
-
-        tx=(*toolpoints)[j].x / maxcoord * MULSHIFT;
-        ty=(*toolpoints)[j].y / maxcoord * MULSHIFT;
-        tz=(*toolpoints)[j].z / maxcoord * MULSHIFT;
-        
-		translate(tx,ty,tz,ident,&scr);
-
-        px2=scr.x;
-        py2=scr.y;
-
-		gl_line(px3,py3,px2,py2,255);
-        }
-}
-*/
-
 void xchg (int *a, int *b)
 {
     int t = *a;
@@ -525,9 +518,12 @@ void TD_drawsurface (TD_Solid * s, int which)
 	    y4 = temp[k + w].y;
 	    c4 = temp[k + w].c;
 
-	/*draw with two triangles */
+	    /*draw with two triangles */
 
-	    if (furthest & 1) { /*draw with hypotenuse from point 1 to point 3*/
+
+
+
+	    if (furthest & 1) {	/*draw with hypotenuse from point 1 to point 3 */
 		if (s->option_flags & TDOPTION_FLAT_TRIANGLE) {
 		    c1 = (c1 + c2 + c3 + c4) >> 2;
 		    (*ds) (x1, y1, x2, y2, x3, y3, c1, clockwise);
@@ -536,7 +532,7 @@ void TD_drawsurface (TD_Solid * s, int which)
 		    (*dt) (x1, y1, c1, x2, y2, c2, x3, y3, c3, clockwise);
 		    (*dt) (x1, y1, c1, x3, y3, c3, x4, y4, c4, clockwise);
 		}
-	    } else { /*draw with hypotenuse from point 2 to point 4*/
+	    } else {		/*draw with hypotenuse from point 2 to point 4 */
 		if (s->option_flags & TDOPTION_FLAT_TRIANGLE) {
 		    c1 = (c1 + c2 + c3 + c4) >> 2;
 		    (*ds) (x1, y1, x2, y2, x4, y4, c1, clockwise);
@@ -563,8 +559,6 @@ void TD_drawsurface (TD_Solid * s, int which)
 }
 
 
-
-
 int compare (const void *vp, const void *vq)
 {
     const int *p = vp;
@@ -581,44 +575,53 @@ struct disttype {
 
 void TD_draw_solid (TD_Solid * s)
 {
-    int n = s->num_surfaces, w, l, i, render;
+    int n = s->num_surfaces, w, l, i, j, render, num_existing_surfaces;
     long max = 0;
 
-    struct disttype *sortarray;
+    struct disttype *sortarray = NULL;
+    temp = NULL;
+
+    gl_trisetdrawpoint(s->draw_point);
 
     if ((sortarray = malloc (s->num_surfaces * sizeof (struct disttype))) == NULL) {
 	fprintf (stderr, "1. Error allocating memory.\n");
-	return;
+	goto fin;
     }
     if (s->option_flags & TDOPTION_INIT_ROTATION_MATRIX)
 	TD_calc_rotation_matrix (s);
 
-    for (i = 0; i < n; i++) {
-	sortarray[i].number = i;
-
-	w = s->surf[i].w;
-	if (max < w)
-	    max = w;
-	l = s->surf[i].l;	/*find the largest surface */
-	if (max < l)
-	    max = l;
+    for (j = 0, i = 0; i < n; i++) {
+	if((s->surf[i].point)) {
+	    sortarray[j++].number = i;
+	    w = s->surf[i].w;
+	    if (max < w)
+		max = w;
+	    l = s->surf[i].l;	/*find the largest surface */
+	    if (max < l)
+		max = l;
+	}
     }
 
+    num_existing_surfaces = j;
+    if(!num_existing_surfaces) goto fin;
+
     if (s->option_flags & TDOPTION_SORT_SURFACES) {
-	for (i = 0; i < n; i++) {
-	    sortarray[i].distance =
+	for (j = 0, i = 0; i < n; i++) {
+	if((s->surf[i].point)) {
+	    sortarray[j++].distance =
 		TD_finddistance (s, &s->surf[i].point[s->surf[i].w / 2 
 				+ s->surf[i].w * (s->surf[i].l / 2)]);
 	    /*the distance of the middle point of the surface */
+	    }
 	}
-	qsort (sortarray, n, sizeof (struct disttype), compare);
+	qsort (sortarray, num_existing_surfaces, sizeof (struct disttype), compare);
     }
 
     max++;
 
     if ((temp = malloc (max * max * sizeof (TD_Short_Point))) == NULL) {
 	fprintf (stderr, "2. Error allocating memory.\n");
-	return;
+	goto fin;
     }
     if (s->option_flags & TDOPTION_ROTATE_OBJECT) {
 	s->x_cam = 0;
@@ -629,13 +632,8 @@ void TD_draw_solid (TD_Solid * s)
 	s->s_cam = 0;
     }
 
-/*    if (s->option_flags & TDOPTION_32BIT_SURFACES) {
-   s->
- */
-
-
-    for (i = 0; i < n; i++) {
 
+    for (i = 0; i < num_existing_surfaces; i++) {
 	if (s->option_flags & TDOPTION_ALL_SAME_RENDER)
 	    render = s->render;
 	else
@@ -661,7 +659,11 @@ void TD_draw_solid (TD_Solid * s)
 	}
     }
 
-    free (temp);
-    free (sortarray);
+    fin:
+
+    if(temp)
+	free (temp);
+    if(sortarray)
+	free (sortarray);
 
 }
